#include <Wire.h>
#include <EmonLib.h>                   // Include EmonLib Library
#include <U8g2lib.h>

EnergyMonitor emon1;                   // Create an instance for energy monitoring
U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, /* reset=*/ U8X8_PIN_NONE);  // Create an instance for OLED display

// Variables for energy calculation
double realPower = 0;                  // Variable to store real power (Watts)
double supplyVoltage = 230.0;          // Assuming a 230V supply voltage (adjust according to your region)
double current = 0;                    // Variable to store current (mA)
double energy = 0;                     // Variable to store total energy usage (kWh)
unsigned long lastUpdateTime = 0;
unsigned long updateInterval = 1000;   // Update interval in milliseconds
double unitRate = 0;                   // Unit rate (cost per kWh)
double totalBill = 0;                  // Variable to store total bill

void setup() {
  Serial.begin(115200);

  // Initialize the OLED display
  u8g2.begin();

  // Initialize the SCT-013 current sensor
  emon1.current(10, 60.6);             // Current: input pin 10, calibration for SCT-013

  // Prompt user to enter the unit rate
  Serial.println("Enter the unit rate (cost per kWh): ");
  while (Serial.available() == 0) {
    // Wait for user input
  }
  unitRate = Serial.parseFloat();
  Serial.print("Unit rate set to: ");
  Serial.print(unitRate);
  Serial.println(" per kWh");
}

void loop() {
  // Calculate real power and energy usage
  current = emon1.calcIrms(1480) * 1000;  // Calculate current in mA
  realPower = current * supplyVoltage / 1000; // Calculate real power in Watts

  // Calculate energy usage
  unsigned long currentTime = millis();
  if (currentTime - lastUpdateTime >= updateInterval) {
    // Energy (kWh) = Power (W) * Time (h)
    energy += (realPower * (updateInterval / 1000.0 / 3600.0));
    lastUpdateTime = currentTime;
  }

  // Calculate total bill
  totalBill = energy * unitRate;

  // Display readings on OLED
  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_open_iconic_all_2x_t);
  u8g2.drawGlyph (1,22, 196);
  
  u8g2.setFont(u8g2_font_ncenB08_tr);
  u8g2.setCursor(22, 12);
  u8g2.print("Curr:");
  u8g2.print(current);
  u8g2.print(" mA");
  u8g2.setCursor(22, 26);
  u8g2.print("RP:");
  u8g2.print(realPower);
  u8g2.print(" W");
  
  u8g2.setFont(u8g2_font_pxplusibmvga9_tf);
  u8g2.setCursor(0, 42);
  u8g2.print("Ener");
  u8g2.print(energy);
  u8g2.print(" kWh");
  
  u8g2.setFont(u8g2_font_pxplusibmvga9_tf);
  u8g2.setCursor(0, 58);
  u8g2.print("Bill:");
  u8g2.print(totalBill);
  u8g2.sendBuffer();

  delay(1000);  // Delay for a second before the next reading
}
